 /*
  * hashblist.h 
  */
  

/* Reply codes */
#define RCODE_NOERROR   0
#define RCODE_FORMERR   1
#define RCODE_SERVFAIL  2
#define RCODE_NXDOMAIN  3
#define RCODE_NOTIMPL   4
#define RCODE_REFUSED   5
#define RCODE_YXDOMAIN  6
#define RCODE_YXRRSET   7
#define RCODE_NXRRSET   8
#define RCODE_NOTAUTH   9
#define RCODE_NOTZONE   10


struct dnsnode {
	char *dnsname;			/* null-terminated */
	int dnslen;			/* length of dns name */
	unsigned int type;	        /* dns record type */		
	unsigned int count;		/* if in top list, # of times seen */
	struct dnsnode *next;


	char *tag;   /* pointer to tag in stringlist */
};

#define HSIZE 104729

#include "stringstore.h"


struct dnslist {
	/* stats */
	unsigned int rcode_counters[16];
	unsigned int dnsrequests;
	unsigned int dnsresponses;
	/* these are taken from responses
	*/
	unsigned int acount;
	unsigned int aaaacount;
	unsigned int mxcount;
	unsigned int ptr;
	int topn_count;
	char fileName[1024];
	int blist_count;		/* number of hashed blacklist entries */
	struct stringlist sl;
	struct stringlist slqueries; /* used for storage of dns answers */
	struct stringlist sltags;	/* used for storage of tag messages */
	int match;			/* need an elog message */
	struct dnsnode *hashlist[HSIZE];	/* blacklist hashlist */
	/*
	struct dnsnode *tnhashlist[HSIZE];      /* topn hashlist */
	unsigned int entries;			/* entry count in tnhashlist */
};

extern int dns_config;
/**************************************************/
/* dns packet info
*/
struct dns_header {
        unsigned short id;           /* query identification number */
        unsigned char  flags1;       /* first byte of flags */
        unsigned char  flags2;       /* second byte of flags */
        unsigned short qdcount;      /* number of question entries */
        unsigned short ancount;      /* number of answer entries */
        unsigned short nscount;      /* number of authority entries */   
        unsigned short arcount;      /* number of resource entries */   
}; 


struct dnsFlowNode {
	
	unsigned short srcPort; 		/*response source port*/
	unsigned short dstPort; 		/*response desination port*/
	unsigned int srcIpAddress;		/*response source address*/
	unsigned int dstIpAddress;		/*response desination address*/
	unsigned short protocol;		/*tcp or udp*/
	unsigned int receiverHits;		/*tally up responses to a host*/
				 
	unsigned int type;	        /* dns record type */	
	unsigned short rcode_counters; /*error code counters*/

	
	char *anquery; /*pointer to question*/

	struct dnsFlowNode *next;
};

struct dnsKey {
	unsigned int ipdst;
	unsigned int ipsrc;
	unsigned short srcport;
	unsigned short dstport;
	unsigned short ipproto;
};

#define HSIZE 104729

struct dnsFlowList{
	unsigned int entries;
	struct dnsFlowNode *last;
	struct dnsFlowNode *hashlist[HSIZE];
};

/**************************************************/
/* function prototypes */

/* insert dns blacklist entries into the hash list
*/


void
config_dnslist(char *tag, char *filename);

/* search a candidate DNS packet for a blacklist match
* and/or compute DNS metrics
*/
void
search_dns(struct t_entry *te);

/* the dns hash list filter may issue a elog probe event to alert the admin
 * that there has been a hit on the dns list.
 * It may also capture dns packet counts of various kinds.
 * All output goes into the mon.lite file.
 */
void 
print_dns(FILE *mon);




