#! /usr/bin/perl -T

#
# cctodns.pl
#
# syntax:
#
#       cciptodb  inputfile  > outputfile
#
#       inputfile - REN-ISAC list of DNS names
#       outputfile - a dns config file suitable for inclusion
#               in a dns server

use strict;
use Socket;
use Getopt::Std;
use File::Basename();

our $input = "/home/mrourmon/etc/dns.orig.txt";
our $zoneopts = 
          "\t allow-transfer { };\n"
        . "\t notify no;\n"
        . "\t also-notify { };\n"
        . "\t allow-query { 131.252.0.0/16; 38.100.216.0/21; 38.100.210.0/23;\n"
	. "\t	127.0.0.0/8; ::1; 2001:0468:1f04:0200::/56; 2610:0010::/32; };\n";



our %dnsdb;
our %options=();
getopts("dw:z:", \%options);
# like the shell getopt, "z:" means d takes an argument
print STDERR "Unprocessed by Getopt::Std:\n" if $ARGV[0];
foreach (@ARGV) {
	print "$_\n";
	}

our $debug;
if (exists $options{'d'}) {
print STDERR "debugging\n";
       $debug = 1;
	}

our $zonefile;
if (exists $options{'z'}) {
# This sad statement is needed because it seems Perl's getopt is broken
# It can't notice an option with a required (but missing) arg.
	die "missing parameter for zone file name, stopped" if ( $options{'z'} =~ /^-/  || $options{'z'} eq "");
	$zonefile = $options{'z'};
	}
else {
# If not stated otherwise, use this name
	$zonefile = "db.badhosts";
print STDERR "Using \"$zonefile\" as default zone file parameter\n" if ($debug);
	}

our $whitelist;
if (exists $options{'w'}) {
	die "missing parameter for whitelist regex, stopped" if ( $options{'w'} =~ /^-/ || $options{'w'} eq "");
	$whitelist = $options{'w'};
	}
else {
	$whitelist = '\.pdx\.edu$';
print STDERR "Using \"$whitelist\" as default whitelist parameter\n" if ($debug);
	}


doit($input,$zonefile);

exit(0);

#
# doit
# parse input file
#
#       1. parse and store syn tuple - file to variables
#       2. compute work weight for this tuple.
#       3. filter netmask/workweight to leave tuples out if desired
#       4. compute a few global stats
#       5. call writeSynReport to compute rs string
#       6. write out long stats (stdout) per tuple stats string in non-summarized mode.
#
sub doit() {

        my $inputfile = shift;
	my $zonefile = shift;
        my $curline;
        my @words;

#        print "# zone \"rogue.example.net\" {\n";
#        print "#\t type master;\n";
#        print "#\t file \"/etc/db.badname\";\n";
#        print "#\t };\n";

        #my $basename = File::Basename::basename($inputfile);
        #my @PARTS = split(/\./, $basename);

        open(FILE, $inputfile) || die "can't open $inputfile: $!\n";

        my $linecount = 0;
        while(<FILE>) {

                $linecount++;
                s/\n//g;
#               s/ //g;
                s/\t//g;
                my $curline = $_;
                # note the escape needed
                #@words = split(/\|/, $curline);
                @words = split(" ", $curline);
#if( $words[0] =~ /^#.*/){
#       print "true\n";
#}
#else {
#       print "false\n";
#}
                if ($linecount < 3) {
                        next;
                }
                elsif( $words[0] =~ /^#/){
                        next;
                }
                elsif( $words[0] =~ /<\/p.*/){ # what is this for?
                        next;
                }
#print "$words[0], $words[1], $words[2]\n";
#print "$words[0]\n";

                my $dnsname = $words[0];
#print "$words[0], $words[1], $words[2]\n";
#print "$words[0]\n";

		if ( valid_domain($dnsname, $whitelist) ) {
                	iptodns($dnsname, $zonefile);
		}
		else {
			print STDERR "$dnsname has been rejected\n";
		}

        }  # end loop
        close(FILE);
}


sub valid_domain
{
	my $domainname = shift;
	my $whitelist = shift;
	my @domain_parts;
	my $sublen;

	return 0 if ( length($domainname) > 255 ); # overall domain name too long

	@domain_parts = split(/\./, $domainname);

	foreach (@domain_parts) {
		$sublen = length;

print STDERR "|$_| $sublen\n" if ($debug);

		return 0 if ( $sublen < 1 || $sublen > 63 );
			# subdomain part incorrect length

		return 0 if (/^-|-$/);		
			# not valid to start or end a subdomain part with a '-'

		s/[-a-zA-Z0-9]+//g;		
			# empty the string of all normally valid characters
		return 0 if ( $_ ne "" );	
			# apparently something was left behind so reject it
	}

	foreach (split(/;/, $whitelist)) {	
		# reject any domain matching a semicolon separated list of regexes
		return 0 if ($domainname =~ /$_/);
	}

	return 1;	# call it good
}

sub
iptodns
{
        my $dnsname = lc $_[0];
        my $zonefile = $_[1];
        my $key;
        my $value;
        my $targetip = "127.0.0.1";
        my $ttl = "30";

        if ($dnsdb{$dnsname}) {
                $dnsdb{$dnsname}++;
        }
        else {
                $dnsdb{$dnsname} = 1;
		printf "zone \"%s\" {\n\ttype master; file \"%s\";\n%s};\n",
			$dnsname, $zonefile, $zoneopts;
        }
}
