#! /usr/bin/perl

#
# httptosnort.pl - extract various fields including URI from RI
#	http feed and create a snort rules file:
#
# syntax:
#
#	httptosnort  -n network inputfile  > stdout
#
#	inputfile - REN-ISAC http list 
#	stdout - snort rules file
#	-n network is required and the network should be snort format.
#		This is the HOME_NET
#
# error handling: 
#  	if address is 0.0.0.0, we ignore it as we only do IP addresses.
#
# note:
#    	default port is 80, but url may have some other port in it.
#    	if so we pluck it out of url.
#
# plan:
# get the IP address
# get the port number
# get the uri
# note: may be NULL, if so leave it out
# snort: need home definition, need http_inspect on
#	TBD: note how to turn off annoying http_inspect discoveries
# SID: base+offset
# rev: 1
#variables:
#snort: $HOME_NET
#$IPADDR
#$PORT 80
#$MSG  RI http feed: and "the path/uri portion"
#$URI  path or null
#$SID  basesid+1
#
# output model:
#
#alert tcp $HOME_NET any -> $IPADDR $PORT (msg:"$MSG1:$MSG2";  uricontent:"/index.html"; nocase;  classtype:web-application-activity; sid:11111; rev:1;)

$HOMENET="";		# put in snort HOME_NET format
$SID=1111111;  
$PORT=80;  
$IPADDR="";
$MSG1 = "RI http feed: ";
$URI="";

use Socket;
use Getopt::Std;
use File::Basename();
#use Time::Local;
# comment in for ctime, note that it overrides localtime
#use Time::localtime;

$debug = 0;
$argc=$#ARGV+1;

if ($argc != 3) {
	usage();
	exit(1);
}

my $input = $ARGV[$argc-1];

use vars qw/ %opt /;
use Getopt::Long;

GetOptions(
        "h" => \$opt{help},
        "h" => \$opt{h},
	"n:s" => \$opt{n},
);

if ( $opt{'h'} ) {
	usage();
	exit(1);
}

# override ips/channel event log message number
if ( $opt{'n'} ) {
	$net = $opt{'n'};		# MAGIC heuristic Konstant
	$HOMENET = "[$net]";
	#print "$net\n";
}
else {
	usage();
	exit(1);
}


doit($input);

exit(0);

#
# doit
# parse input file
# throw out extraneous bits and extract the good parts
#
sub doit() { 
	
	my $inputfile = $_[0];
	my $curline;
	my @words;
	my $ipaddr;
	my $port;

	#my $basename = File::Basename::basename($inputfile);
	#my @PARTS = split(/\./, $basename);

	open(FILE, $inputfile) || die "can't open $inputfile: $!\n";

	my $linecount = 0;
	while(<FILE>) {

		$linecount++;
		s/\n//g;
		s/ //g;
		s/\t//g;
		my $curline = $_;
		# note the escape needed
		@words = split(/\|/, $curline);
		if ($linecount < 3) {
			next;
		}
		elsif( $words[0] =~ /^#.*/){
			next;
		}
                elsif( $words[0] =~ /<\/p.*/){
			next;
		}

		$ipaddr = $words[3];
		# 80 is default
		$port = 80;
		# but get the port out of the uri if there is one
		$uri = $words[7];
		# stupid but it works, regex better?
		$r1 = substr($uri, index($uri, '/')+1);
		$r2 = substr($r1,index($r1, '/')+1);
		$r3 = substr($r2,index($r2, '/'));
		@uriwords = split(/\//, $uri);
		@uriaddr = split(/:/, $uriwords[2]);
		if ($uriaddr[1]) {
			$port = $uriaddr[1];
		}
		#print "DEBUG: $uri (p=$port)\n";
  		#print "ipaddr $ipaddr $port\n";

		printSnortRule($ipaddr, $port, $r3, $uri);

	}  # end loop
	close(FILE);
}

sub 
printSnortRule 
{
	my $ip = $_[0];
	my $port = $_[1];
	my $path = $_[2];
	my $uri = $_[3];

	# TBD. fix this because possibly one IP addr has many uris
	# but that doesn't seem to be the case with the data.
	#
	# we saw it before ignore it
	if ($ipaddrs{$ip} eq $port) {
		return;
	}
	$ipaddrs{$ip} = $port;

	# this means they couldn't figure out the IP address
	# so we won't use it
	if ($ip eq "0.0.0.0") {
		return;
	}
	
	print("alert tcp $HOMENET any -> $ip $port ");
	print ("(msg:  \"$MSG1:$uri\";");
	print ("uricontent: \"$path\";");
	print ("nocase; classtype:web-application-activity; sid:$SID;");
	print ("rev:1;)");
	print ("\n");

	$SID++;
}

sub usage
{
	print "script -n network input > stdout\n";
	print "-n net/mask,net/mask required\n";
}
