#! /usr/bin/perl

#
# daily.pl
#
# syntax:
#	# daily.pl base_log_dir
# e.g.,
#	# daily.pl /home/mrourmon/logs
#
# run at around midnight to start log directory over for next day.
# run from crontab.
# 0	0	*	*	*	root	/home/mrourmon/bin/daily.pl /home/mrourmon/logs
#
# All log dirs are found under a base log directory with the following dir scheme:
#
# /home/mrourmon/logs
#	Mon etc (topn_ip) are put in daily directories at the top of the logdir
#	mon.lite
#	tworm
#	topn
#	portreport
#	p2preport
#	rawirc
#	irc
#	udpreport
#	rawpotdump
#	rawdnsdump
#	syndump
#	email
#	rawemail
#	rawssh
#	ssh
#	etc.
#
#	each directory above has Sun Mon Tue ... Sat daily directories
#
# This perl script is called once at midnight to prepare for the next
# day's logging.
#
# it's job is to zero out the daily (Sun Mon Tue Wed Thu Fri Sat)
# directory in the base logging directory for all 3 of the above.
#
# for the topn logging directory it also creates a symlink
# to a cur for the current day.
#
# Assumption: output_dir has subdirs for days of week.  Mon, Tue, etc.
# Thus output format is:  log/Mon/date.mon.lite
# form is thus:
# Sun Mon Tue Wed Thu Fri Sat 

$debug = 0;
$argc=$#ARGV+1;

if ($argc != 1) {
	print "syntax: # daily.pl output_dir\n";
	print "example: # daily.pl /home/mrourmon/logs\n";
	exit(1);
}
use File::Basename();

use Time::Local;
# comment in for ctime, note that it overrides localtime
use Time::localtime;

my $outputdir = $ARGV[0];

################################################### 
$start_time=`date`;
$start_time=~ s/\n//g;		# zap the newline at the end of the date
@time = split(/ /, $start_time);
$dow = $time[0];
$yesterday = get_yesterday($dow);

# create a timestamp file name/date
$datestamp = `date +%Y-%m-%d`;
chomp($datestamp);
$datefile = "LOG_DATE" . "\." . $datestamp . "\." . "txt";

chdir($outputdir);

# zap the event database so we start over */
system("rm -fr eventdb.db");   

# topn daily prep
system("rm -fr $dow"); 
system("rm -f topn_today"); 
system("mkdir $dow"); 
system("ln -s $dow topn_today");

# put the date for the day as a file in
# the topn_ip directory for possible log backup
chdir ("$dow");
system("echo $datestamp > $datefile");
chdir ("..");

# mon.lite daily prep
system("rm -fr mon.lite/$dow"); 
system("rm -fr mon.lite/mon_today"); 
system("mkdir mon.lite/$dow"); 
system("ln -s $dow mon.lite/mon_today");

# tworm daily prep
system("rm -fr tworm/$dow"); 
system("rm -fr tworm/tworm_today"); 
system("mkdir tworm/$dow"); 
system("ln -s $dow tworm/tworm_today");

# portreport daily prep
system("rm -fr portreport/$dow"); 
system("rm -f portreport/portreport_today"); 
system("mkdir portreport/$dow"); 
system("ln -s $dow portreport/portreport_today");

# p2preport daily prep
system("rm -fr p2preport/$dow"); 
system("rm -f p2preport/p2preport_today"); 
system("mkdir p2preport/$dow"); 
system("ln -s $dow p2preport/p2preport_today");

# rawirc daily prep
system("rm -fr rawirc/$dow"); 
system("rm -f rawirc/rawirc_today"); 
system("mkdir rawirc/$dow"); 
system("ln -s $dow rawirc/rawirc_today");

# irc daily prep
system("rm -fr irc/$dow"); 
system("rm -f irc/irc_today"); 
system("mkdir irc/$dow"); 
system("ln -s $dow irc/irc_today");

# udpreport daily prep
system("rm -fr udpreport/$dow"); 
system("rm -f udpreport/udpreport_today"); 
system("mkdir udpreport/$dow"); 
system("ln -s $dow udpreport/udpreport_today");

# psuportreport daily prep
#system("rm -fr psuportreport/$dow"); 
#system("rm -f psuportreport/portreport_today"); 
#system("mkdir psuportreport/$dow"); 
#system("ln -s $dow psuportreport/portreport_today");

# rawp2p daily prep
system("rm -fr rawp2p/$dow"); 
system("rm -f rawp2p/rawp2p_today"); 
system("mkdir rawp2p/$dow"); 
system("ln -s $dow rawp2p/rawp2p_today");

# full/rawsynreport daily prep
#system("gzip -r rawsyndump/$yesterday &");
system("rm -fr rawsyndump/$dow");
system("rm -f rawsyndump/rawsyndump_today");
system("mkdir rawsyndump/$dow");
system("ln -s $dow rawsyndump/rawsyndump_today");

# backed synreport daily prep
#system("gzip -r synreport/$yesterday &");
system("rm -fr syndump/$dow");
system("rm -f syndump/syndump_today");
system("mkdir syndump/$dow");
system("ln -s $dow syndump/syndump_today");

# full/rawemail daily prep
system("rm -fr rawemail/$dow");
system("rm -f rawemail/rawemail_today");
system("mkdir rawemail/$dow");
system("ln -s $dow rawemail/rawemail_today");

# full/email daily prep
system("rm -fr email/$dow");
system("rm -f email/email_today");
system("mkdir email/$dow");
system("ln -s $dow email/email_today");

# full/rawipmesh daily prep
#system("rm -fr rawipmesh/$dow");
#system("rm -f rawipmesh/rawipmesh_today");
#system("mkdir rawipmesh/$dow");
#system("ln -s $dow rawipmesh/rawipmesh_today");

# full/rawpotdump daily prep
system("rm -fr rawpotdump/$dow");
system("rm -f rawpotdump/rawpotdump_today");
system("mkdir rawpotdump/$dow");
system("ln -s $dow rawpotdump/rawpotdump_today");

# ssh daily prep
system("rm -fr ssh/$dow");
system("rm -f ssh/ssh_today");
system("mkdir ssh/$dow");
system("ln -s $dow ssh/ssh_today");

# rawssh daily prep
system("rm -fr rawssh/$dow");
system("rm -f rawssh/rawssh_today");
system("mkdir rawssh/$dow");
system("ln -s $dow rawssh/rawssh_today");

# rawudpreport daily prep
system("rm -fr rawudpreport/$dow"); 
system("rm -f rawudpreport/rawudpreport_today"); 
system("mkdir rawudpreport/$dow"); 
system("ln -s $dow rawudpreport/rawudpreport_today");

# rawdns prep
if (-e "rawdns") {
	system("rm -fr rawdns/$dow"); 
	system("rm -f rawdns/rawdns_today"); 
	system("mkdir rawdns/$dow"); 
	system("ln -s $dow rawdns/rawdns_today");
}

exit(0);

# end of main

sub get_yesterday
{
	my $today = shift;
	if ($today eq "Mon") {
		return("Sun");
	}
	elsif ($today eq "Tue") {
		return("Mon");
	}
	elsif ($today eq "Wed") {
		return("Tue");
	}
	elsif ($today eq "Thu") {
		return("Wed");
	}
	elsif ($today eq "Fri") {
		return("Thu");
	}
	elsif ($today eq "Sat") {
		return("Fri");
	}
	elsif ($today eq "Sun") {
		return("Sat");
	}
}


