#! /usr/bin/perl

# change as appropriate
my $TARCOMMAND="/usr/bin/tar -czf ";

# ----------------------

#
# logbackup.pl
#
# syntax:
#	# logbackup.pl base_log_dir  base_log_backup_dir
# e.g.,
#	# logbackup.pl /home/mrourmon/logs  /home/mrourmon/logstash
#
# run AFTER midnight say at 12:05 to backup previous day's logs
# run from crontab.
# 5	0	*	*	*	root	/home/mrourmon/bin/logbackup.pl /home/mrourmon/logs  /home/mrourmon/logstash
# 
# log backup for ourmon
# basic idea:
# backup all of the log files for the day BEFORE today
# e.g., it's midnight Sunday, Monday is next
#	daily.pl runs which destroys the previous Mon and setups the
#	new Mon
# therefore we backup the previous day Sunday and we don't care
#	how long that takes (well it had best be done before the
#	next Sunday).
# step 1: 
#	compute previous day
#	build logfile name
#	run tar on input arg

$debug = 0;
$argc=$#ARGV+1;

if ($argc != 2) {
	print "syntax: # logback.pl log_dir logstash_dir\n";
	print "example: # logback.pl /home/mrourmon/logs /home/mrourmon/logstash\n";
	exit(1);
}
use File::Basename();

use Time::Local;
# comment in for ctime, note that it overrides localtime
use Time::localtime;

my $logdir = $ARGV[0];
my $stashdir = $ARGV[1];

################################################### 
$start_time=`date`;
$start_time=~ s/\n//g;		# zap the newline at the end of the date
@time = split(/ /, $start_time);
$dow = $time[0];
$yesterday = get_yesterday($dow);

# first get the contents of the previous day's timestamp
# this is the date stamp for our backup log file name
# create a timestamp file name/date
# $datefile = "LOG_DATE" . "\." . $datestamp . "\." . "txt";

chdir($logdir);
chdir($yesterday);
$datestamp = `cat LOG_DATE*`;
chomp($datestamp);
if ($debug) {
	print "--->$datestamp<---\n";
}

# construct the basename of our tar file
$basename = $stashdir . "\/" .  $datestamp . "\." . "tar.gz";
if ($debug) {
	print "$basename\n";
}

# 
#tar czvf ../logstash/`date +%Y-%m-%d`.tar.gz Wed */Wed
$tarcommand = $TARCOMMAND . "$basename" . " " .  "$yesterday" . " " . "*/$yesterday";

if ($debug) {
	print "$tarcommand\n";
}

chdir($logdir);
system("$tarcommand");

exit (0);

# end of main

sub get_yesterday
{
	my $today = shift;
	if ($today eq "Mon") {
		return("Sun");
	}
	elsif ($today eq "Tue") {
		return("Mon");
	}
	elsif ($today eq "Wed") {
		return("Tue");
	}
	elsif ($today eq "Thu") {
		return("Wed");
	}
	elsif ($today eq "Fri") {
		return("Thu");
	}
	elsif ($today eq "Sat") {
		return("Fri");
	}
	elsif ($today eq "Sun") {
		return("Sat");
	}
}



