#!/usr/bin/perl

use Socket;
my $LOOKUP_LIMIT = 1000;

if ($#ARGV < 1)
{
 print_usage();
 exit();
}

open(BLACKLIST, "<", $ARGV[0]) or die $!;
dbmopen(my %bldb, $ARGV[1], 0666);
my @ips;
while (<BLACKLIST>)
{
 chomp;
 push(@ips, $_);
}
lookupCountries(@ips);
foreach $ip (@ips)
{
	$bldb{$ip} = $countryByIp{$ip}."s";
}

sub print_usage()
{
 print "ssh-db.pl <blacklist filename> <db filename>\n";
}

sub insubnet
{
        my $netmask = $_[0];
        my $ip = $_[1];
        my $hexmask;
        my $rnet1;
        my $rnet2;
        my $net1;
        my $net2;
        my $index=0;

        my @nets = split(",", $netmask);

        while ($nets[$index]) {

                my @words = split("\/", $nets[$index]);
                my $subnet = $words[0];
                my $mask = $words[1];

                # fix subnet if too short - we need full dot quad notation
                my @quads=split('\.',$subnet);
                my $suffixquads = 3 - $#quads;  # number of .zero quads needed at end
                # append enough dot quads if needed
                if ($suffixquads == 3) {
                        $subnet = $subnet . ".0.0.0";
                }
                elsif ($suffixquads == 2) {
                        $subnet = $subnet . ".0.0";
                }
                elsif ($suffixquads == 1) {
                        $subnet = $subnet . ".0";
                }

                # if slash notation
                # 1. convert the specified mask
                # to a hex value (a real mask)
                $hexmask = 0xffffffff << (32 - $mask);
                # limit to 32 bits please
                $hexmask = $hexmask & 0xffffffff;

                # ip address based netmask as opposed to slash
                #$hexmask = unpack('N', inet_aton($mask));

                # 2. convert the ipaddr to a host int
               $net1 = unpack('N', inet_aton($ip));
                $net2 = unpack('N', inet_aton($subnet));
                # printf ("inputip: %x inputnet %x mask %x\n", $net1, $net2, $hexmask);

                # 3. compare the results
                $rnet1 = $net1 & $hexmask;
                $rnet2 = $net2 & $hexmask;

                # print "$rnet1, $rnet2:", "$rnet1 == $rnet2\n";
                if ($rnet1 == $rnet2) {
                        return (1);
                }
                $index++;
        }
        return(0);
}


sub lookupCountries
{
        foreach my $ip ( @_ ) {
                if ($DEBUG) { print STDERR "lookupCountries: --> $ip\n"; }

                if ($countryByIp{$ip}) {
                        # Skip it if it's already in the cache.
                        if ($DEBUG) { print STDERR "Cache hit $ip\t-> $countryByIp{$ip}\n"; }
                }
                # Don't lookup home network IPs.
                elsif ($homeNetmaskFlag && insubnet($homeNetmask, $ip)) {
                        $countryByIp{$ip} = 'home';
                        if ($DEBUG) { print STDERR "Skipped $ip\t-> home network\n"; }
                }
                # Don't lookup private IPs.
                elsif (insubnet('10/8', $ip) || insubnet('172.16/12', $ip) || insubnet('192.168/16', $ip)) {
                        $countryByIp{$ip} = 'private';
                        if ($DEBUG) { print STDERR "Skipped $ip\t-> private network\n"; }
                }
                # We don't want to kill Cymru with lookups in case of a DDOS.
                elsif ($lookups >= $LOOKUP_LIMIT) {
                        if ($lookups == $LOOKUP_LIMIT) {
                                print STDERR 'SSH country code lookup exceeded the limit: '.$LOOKUP_LIMIT."\n";
                        }
                }
                else {
                        # Query the Cymru system for a country code.
                        my $query = join('.', reverse(split(/\./, $ip))) .'.origin.asn.cymru.com';
                        if ($DEBUG) { print STDERR "params to dig +short $query TXT"; }
                        my $result = `dig +short $query TXT`;
                        chomp($result);
                        $lookups++;

                        # The results should have one or more lines like this:
                        #   "6366 | 131.252.0.0/16 | US | arin | 1989-02-13"\n
                        # we'll just use the first one.
                        my @lines = split("\n", $result);
                        if (scalar @lines) {
                                # Trim off the quotes then split it up by the pipes.
                                $lines[0] =~ s/\"//g;
                                my @parts = split(/ \| /, $lines[0]);
                                if ($DEBUG) { print STDERR "Looked up $ip\t-> $parts[2]\n"; }
                                $countryByIp{$ip} = $parts[2];
                        }
                        else {
                                $countryByIp{$ip} = '??';
                                if ($DEBUG) { print STDERR "Error looking up $ip.\n"; }
                        }
                }
        }
}


