#! /usr/bin/perl

#
# may be run:
#	per sample
#	per N samples
#
# See the function printUsageAndExit for complete
# syntax. 
#
# TBD: #          
# 	put test cases so that user cannot use certain switches in the 30sec usecase, 
#	and similarly cannot use certain switches in the summarization (-e ) 
#
#############################################################
$maxudpweight = 10000000;

#top 3 ipdsts will be printed for eacg ipsrc
$maxdsts = 5;

#counts of security violations seen
$c1=0;
$c2=0;

#arrays for storing security lists 
@sumWWv;
@sumStrmv;
########################################################


$start_time=`date`;
$start_time=~ s/\n//g;
$stime=time();

use Getopt::Long;
use File::Basename();
use Socket;

my %hosts = ();
my $directory="";
my $delimiter = "\n";

my $ninstances1 = 0;
my $mode = 0;

if ( (scalar @ARGV) < 1 ) { 
	printUsageAndExit(); 
}

GetOptions(
        "help" => \$opt{help},
        "h:s" => \$opt{h},
        "u:s" => \$opt{u},
	"d:s" => \$opt{d},
	"f:s" => \$opt{f},
        "e:s" => \$opt{e},
	"p" => \$opt{p},
	"m" => \$opt{m},
	"t:s" => \$opt{t},
	"w:s" => \$opt{w},
);

$eventflag=0;
$eventfile="";
if ($opt{'e'}) {
        $eventfile = $opt{'e'};      
        $eventflag = 1;                
        #print $eventfile;
}

if ($opt{"help"}) {
	printUsageAndExit();
}


if ($opt{'u'}) {
        $maxudpweight = $opt{'u'};      
        if ($maxudpweight < 0) {
		print "udpww should be greater than zero\n";
		exit;
                # user entered wrong value
        }
}

$tuples =-1;
if ($opt{'t'}) {
	$tuples= $opt{'t'};
	if ($tuples < 1) {
		print ("number of tuples should be atleast 1");
		exit;
	}
}

$wwcheck=0;
if ($opt{'w'}) {
	$ww_stop = $opt{'w'};
	$wwcheck=1;
	if ($ww_stop < 0) {
		print "udpww check should be greater than zero\n";
		exit;
	}
}	

#output based on a net/mask pair.  i.e., ip must be "inside" this network
$home_netmaskflag = 0;
if ($opt{'h'}) {
        $home_netmask = $opt{'h'};    # netmask address should be 10/8 or the like
        $home_netmaskflag = 1;          # capability is on
}

$p2p_only = 0;
if ($opt{'p'}) {
        $p2p_only = 1;
}

# sort by max packets sent/received
$sortbymaxpackt = 0;
if ($opt{'m'}) {
        $sortbymaxpackt = 1;
}

if ( $opt{'d'} ) {
	$directory = $opt{'d'};
	# read all files in a directory into the array and
	# prepend directory path to filename
	#
        $mode=1;
	opendir (DIR, $directory) or 
	    die "can't open $directory directory: $!\n";
	@FILES = readdir DIR;
	closedir DIR;
	$size = scalar @FILES;
	while ($size > -1) {
		# prepend directory path to files
		if ( -f "$directory/$FILES[$size]" ) {
			$FILES[$size] = $directory.'/'.$FILES[$size];
		}
		$size--;
	}
}
 
if ( $opt{'f'} ) {
	@FILES = split /\s+/, $opt{'f'};
	$ninstances1 = scalar @FILES;
}

# files are now in FILES array
# loop thru them
#
$mode_30sec=0;
if(($ninstances1 == 1) && ($mode==0)){
	$mode_30sec=1;
	my $one_file = shift @FILES; 
        open(FILE, $one_file) || die "can't open input file $one_file file: $!\n";
        @FILE = <FILE>;
        close(FILE);

        $phrases = join("<br>",@FILE);
        udpReportOpen() ;
        #  fetch each line from datafile, line is separated by "\n"
        @phrases = split(/$delimiter/,$phrases);

        # walk phrases and process on line by line basis
        foreach my $elem (@phrases) {
                # get rid off blanks, return, line breaks
                $elem =~ s/<br>//g;
         	  # save each item to array @items_in_eachline, from each line,
     		@items_in_eachline = split(/:/,$elem);
                udpReportOneLine(\@items_in_eachline);
        }
exit(1);
}

else {
	#this is the summarisation part
        # with summarization mode, we expect a timestamp in the filename

        $filecount = scalar @FILES;
        while ( $inputfile = shift @FILES ) {
                if ($inputfile eq ".") {
                        next;
                }
                if ($inputfile eq "..") {
                        next;
                }
                #do processing for each file
                doit($inputfile);
        }

        $e1time=time();
        $f1time = $e1time - $stime;
        $eminutes = 0;
        if ($f1time > 0) {
                $eminutes = $f1time/60;
        }
        #print "\nElapsed time between reading all files and before doing sum_report is : $f1time seconds, $eminutes minutes\n";

        # now perform the summing part
	#initialize security list violation lists
        sum_report();

        $etime=time();
        $ftime = $etime - $stime;
	$eminutes = 0;
	if ($ftime > 0) {
		$eminutes = $ftime/60;
	}
        print "\nelapsed time: $ftime seconds, $eminutes minutes\n";
        exit($ftime);
}



#end of main 
#########################################################################################


sub printUsageAndExit()
{
	print "Usage: $0 -d directory -f \"file1 file2 ... fileN\"\n";
	print "\t-help            displays usage\n";
	print "\t-h net/mask \n";	
        print "\t-m for sorting by max packet sent else by defaults sorts by Avg ww \n";
        print "\t-p only for p2p hosts\n";
	print "\t-u UDP threshold\n";
	print "\t-t number of tuples from each file\n";  
	print "\t-e eventfile(only for 30 sec) for udp threshold exceeded events \n";
	print "\t-w udpww     This will  exit the  raw file as soon as you encounter workweight < udpww \n";   
	exit;
}


sub doit()
{
        my $inputfile = $_[0];

        # extract timestamp from file into curts if any.
        #  Fri_Feb_15_16:07:34_PST_2008.udpreport.txt

        my $basename = File::Basename::basename($inputfile);
        my @PARTS = split(/\./, $basename);
        if (check_ts($PARTS[0])) {
                # curts is a global
                $cur_ts = $PARTS[0];
        }

        open(FILE, $inputfile) || die "can't open $inputfile: $!\n";

        # line format:
        # format: ip address_src: ip address_dst:  weight: udp sends: udp recvs: unreach: pings: l3:       l4: udpflags: noports: port tuple (dst_port, count)
        #         0                1                2      3          4           5       6      7          8   9        10       11

        my $count =0;	
        while(<FILE>) {
                $totaltuples++;
                s/\n//g;
                $curline = $_;
                @words = split(/:/, $curline);
                if($words[0] =~ /^#./){
                        next;
                }
                $sysname = "";
                $sysname = gethostip($words[0]);
		if ($tuples > 0){
			if ($count == $tuples){
				return;
			}
		}
                $val=udpReportOneLine(\@words);
	        $count++;
		if ($val==5){
			return;
		}	
        }
}


sub sum_report()
{
	my $ipcount =0;
        my @sortedtuples;

	#calculate the avg udp packets sent for each ip 
	#calculate the avg udpww for each ip 
	
	foreach $ipkey (keys %sumUdpsent) {
                        my $udpsent1 =  ($sumUdpsent{$ipkey} / $sumInstances{$ipkey});
                        $sumUdpsent{$ipkey}=$udpsent1;
			my $udpwwa =  ($sumWorkweight{$ipkey} / $sumInstances{$ipkey});
                        $sumAvgww{$ipkey}=$udpwwa;
        }

	#decide sorting scheme
	if ($sortbymaxpackt == 1 ) {
		@sortedtuples = sort sort_by_max_packt keys(%sumUdpsent);
	} 
	else { 
		@sortedtuples = sort sort_by_av_ww keys(%sumAvgww);
	}
	
	# total number of ip sources seen
	$ipaddrcount = scalar @sortedtuples;

	print "Udp Summarization:\n";
	print "----------------------------------------------\n";
	print "Start time: $start_time\n";
	if ($tuples > 0) {
		print "Only summarizing first $tuples IPs of each file\n";
	} 
	if ($wwcheck==1){
		print "Udpww check restricted to $ww_stop\n";
	} 
	if ($sortbymaxpackt == 1) {
		print("Sorted by max packets sent\n");
        }
	else {
		print("Sorted by average udpww\n");
	}
	if ($p2p_only == 1) {
		print ("Only p2p hosts\n");
	}
	printf ("Total unique IP address seen: %u\n\n", $ipaddrcount);
	printf("-----------------------------------------------------------------------\n");
        printf("%-16.16s (%-12s:%-12s:%-12s) (%8s:%8s) %9s   (%8s:%8s) (%-3s:%-3s:%-3s:%-3s:%-3s:%-3s) (%-5s:%-5s)",   "ipsrc"
,"wwmin","ww", "wwmax", "udpsent", "udprecv", "icmpcount" , "l3count", "l4count", 
                             "s0", "s1","s2","s3","s4","s5","savg","ravg" );
	print("\n");

	printf("\tDNS name if any\n");	
	printf("\tOrred flag summary percentage of b ,g ,d,s flags\n");
	printf("\tGuesstimator for percent p2p, scan and oneway\n");
	printf("\t#instances: firstsample: Last sample\n");
	printf("\tport tuples\n");
	printf("\tTop 5  ipdsts\n\n");
        
	# loop through sorted ip addresses
	my $i = 0;
	for ($i = 0; $i < $ipaddrcount; $i++) {
		my $ipsrc = $sortedtuples[$i];
        	my $dnsname = gethostip($ipsrc);
        	my $instances = $sumInstances{$ipsrc};
	
		#check the following

        	if ($instances == 0) {
                	$instances = 1;
        	}  
        	my $ww = int(($sumWorkweight{$ipsrc}) / $instances);
        	my $wwmax = $sumWorkweightMax{$ipsrc};
        	my $wwmin = $sumWorkweightMin{$ipsrc};
        	my $ap = $sumAppflags{$ipsrc};

        	my $udpsent2 = int ($sumUdpsent{$ipsrc}) ;
		my $udprecv = int(($sumUdprecv{$ipsrc}) / $instances);
		my $icmpcount = int(($sumIcmpcount{$ipsrc}) / $instances);
        	my $l3count = int(($sumL3count{$ipsrc}) / $instances); 
        	my $l4count = int(($sumL4count{$ipsrc}) / $instances); 
        	my $scount =  int(($sumScount{$ipsrc}) /$instances);                
        	my $savg =    int(($sumSavg{$ipsrc}) /$instances);                
        	my $ravg =    int(($sumRavg{$ipsrc}) /$instances);                
        
        	my $size0 = int (($sumSize0{$ipsrc}*1.0) / $instances);
        	my $size1 = int (($sumSize1{$ipsrc}*1.0) / $instances);
        	my $size2 = int (($sumSize2{$ipsrc}*1.0) / $instances);
        	my $size3 = int (($sumSize3{$ipsrc}*1.0) / $instances);
        	my $size4 = int (($sumSize4{$ipsrc}*1.0) / $instances);
        	my $size5 = int (($sumSize5{$ipsrc}*1.0) / $instances);
        
        	my $p2p = int (( $sumGuessp2p{$ipsrc}* 100.0) / $instances) ;
        	my $oneway = int (( $sumGuessoneway{$ipsrc} * 100.0) / $instances);
        	my $scan = int (( $sumGuessscan{$ipsrc} * 100.0)/ $instances);
	
		my $firstTS = $sumFirstTS{$ipsrc};
		my $lastTS = $sumLastTS{$ipsrc};
	
        	my $sbuf = sprintf("#%-16.16s (%-12s:%-12s:%-12s) (%8s:%8s) %9s   (%8s:%8s) (%-3s:%-3s:%-3s:%-3s:%-3s:%-3s) (%-5s:%-5s)",   "$ipsrc","$wwmin","$ww", "$wwmax", "$udpsent2", "$udprecv", "$icmpcount" , "$l3count", "$l4count",
                             "$size0", "$size1","$size2","$size3","$size4","$size5","$savg","$ravg" );
		print $sbuf;
		print "\n";
        	print "\tDNS: $dnsname\n";
		sumflags($ipsrc);
		printf("\tGuesstimator percents: p2p %u , scan %u , oneway %u  \n", $p2p, $scan ,$oneway);
        	print "\t#$instances:  $sumFirstTS{$ipsrc}: $sumLastTS{$ipsrc}: \n";
	
		sumports($ipsrc);
		sumdsts($ipsrc);
		print "\n\n"; 
	}
#	print("--------------------------------------------------\n");
#	print ("Two security lists\n\n");
#	print("1: Total $c1  Udpww threshold violations\n");
#	print("Timestamp				IPaddress	\t	Workweight\n");
#	for(my $k1=0; $k1 < $c1; $k1++){ 
#		print($sumWWv[$k1]);
#	}
#
#	print("\n2: Total $c2    Stormworm instances  reported\n");
#	for(my $k2=0; $k2 < $c2; $k2++){ 
#		print($sumStrmv[$k2]);
#	}
}



$curip1 = 0;
                
sub sumipdstsby_value
{      
        if ($sumIpdst{$curip1}->{$a} < $sumIpdst{$curip1}->{$b}) {
                1;
        }
        elsif ($sumIpdst{$curip1}->{$a} == $sumIpdst{$curip1}->{$b}) {
                0;
        }
        elsif ($sumIpdst{$curip1}->{$a} > $sumIpdst{$curip1}->{$b}) {
                -1;
        }
}
 

    
sub sumdsts
{
        my $ipsrc = $_[0];
        my $portkey;
        my $nps="";
        
        $curip1 = $ipsrc;
         
        my @sp  = sort sumipdstsby_value keys %{$sumIpdst{$ipsrc}};
        my $count1 = scalar @sp;

	#only top 3 destinations
        if ($count1 > $maxdsts ) {
                $count1 = $maxdsts;
        }
        
        for (my $i = 0; $i < $count1; $i++) {
                $nps = $nps . "($sp[$i] : $sumIpdst{$ipsrc}->{$sp[$i]})  ";
	
        }
        print "\tTop [$count1] Ipdsts:$nps\n";
         
}
                   
sub sumflags
{
	my $ipsrc = $_[0];
	my $nps="";

	
	foreach $flagkey (keys %{$sumFlag{$ipsrc} }) {
		my $per = int ((($sumFlag{$ipsrc}->{$flagkey})*100.0)/$sumInstances{$ipsrc});
         	$nps = $nps."$flagkey:$per   ";
	}
	print("\tFlag percents \t $nps\n");
}

              
sub sort_by_max_packt
{
        if ($sumUdpsent{$a} < $sumUdpsent{$b}) {
                1;
        }
        elsif ($sumUdpsent{$a} == $sumUdpsent{$b}) {
                0;
        }
        elsif ($sumUdpsent{$a} > $sumUdpsent{$b}) {
                -1;     
        }
}


                        
sub sort_by_av_ww
{
        if ($sumAvgww{$a} < $sumAvgww{$b}) {
                1;
        }
        elsif ($sumAvgww{$a} == $sumAvgww{$b}) {
                0;
        }
        elsif ($sumAvgww{$a} > $sumAvgww{$b}) { 
                -1;
        }
}
                        


sub udpReportOneLine
{
   my(@items) = @{(shift)};
   my $i = 0 ;

  # udperror_list tuple syntax:
        # update and make png file for topn_udperror
        # format: ip address: weight: udp sends: udp recvs: ETC

                my $udps;
                my $udpip  = $items[$i];
                my $udpww  = $items[$i+1];

         	if(mode_30sec == 0){
			if ($wwcheck) {
				if ($udpww < $ww_stop) {
					return 5;
				}
			}
				
			if($udpww > $maxudpweight){
                		my $sbuf2 = sprintf("%s         %-16.16s              %12s\n",$cur_ts,  "$udpip" , "$udpww");
                        	$sumWWv[$c1] = $sbuf2;
                        	$c1 += 1;
                	}
			if ($home_netmaskflag == 1) {
                		my $rc;
                        	# rc is 1 if in subnet, 0 if not in subnet
                        	$rc = insubnet($home_netmask, $udpip);
                        	#print "$udpip $rc\n";
                        	if ($rc == 0) {
                        		#print "skipping: $udpip $rc\n";
                                	return 1;   #goto next tuple
                        	}
                	}
		}

                my $udpsent = $items[$i+2];
                my $udprecv = $items[$i+3];
                my $unreach = $items[$i+4];     # icmp unreach sent back
                my $icmpcount = $items[$i+5];   # pings sent by ip
                my $l3count   = $items[$i+6];   # count of IP dsts
                my $l4count   = $items[$i+7];   # count of UDP dsts
                my $uappf = $items[$i+8];       # old hw app flags
                my $uappl = $items[$i+9];       # app lower case
                my $uapph = $items[$i+10];      # app upper case
                my $size0 = $items[$i+11];
                my $size1 = $items[$i+12];
                my $size2 = $items[$i+13];
                my $size3 = $items[$i+14];
                my $size4 = $items[$i+15];
                my $size5 = $items[$i+16];

                my $savg = $items[$i+17];
                if ($udpsent) {
                        $savg = int($savg/$udpsent);
                }
                else {
                        $savg = 0;
                }
                my $ravg = $items[$i+18];
                if ($udprecv) {
                        $ravg = int($ravg/$udprecv);
                }
                else {
                        $ravg = 0;
                }

                my $portcount = $items[$i+19];  # number of port tuples
                my $scount = $items[$i+20];     # number of port tuples
                my $udpdst = $items[$i+21]; 
                my $portstring = $items[$i+22]; # all port tuples in one string
		$uapplc=$uappl;
                my $uaf = get_appflag($uappf, $uappl, $uapph);
                if ($udpww > 0) {
			if($mode_30sec == 1){
				$udps = udpReport($udpip, $udpdst, $udpww, $udpsent,
                                $udprecv, $unreach, $icmpcount,
                                $l3count, $l4count, $uaf, $portcount,
                                $size0, $size1, $size2, $size3, $size4, $size5, $scount,
                                $portstring, $savg,$ravg);

				 if (($udpww > $maxudpweight) && ($eventflag)){
					# do some weeding - e.g., try not to report on anomalous p2p activity
                                        storeEvent($eventfile,"udpweight threshold exceeded:\n", $udps);
                        	 }
                        }
			else { #summarization mode
			        $udps = udpReport($udpip, $udpdst, $udpww, $udpsent,
                                $udprecv, $unreach, $icmpcount,
                                $l3count, $l4count, $uaf, $portcount,
                                $size0, $size1, $size2, $size3, $size4, $size5, $scount,
                                $portstring, $savg,$ravg);
				
                        }  #end of summarisation one tuple
                        

               }  #end of if udpww > 0; 
return 1;
}


sub udpReportOpen
{
        my $pathname = $_[0];
        my $sbuf;

        print "topn udp error list: port signatures\n\n";
        $sbuf = sprintf("%-16.16s %-16.16s %-12.12s %-16.16s %8s: %8s: %8s: %9s: %12s: %-5.5s:           %-5.5s:   %9s:  %10s: %s\n\n",
                "ip src:",
                "ip dst:",
                "weight:",
                "guess:",
                "udp_sent",
                "udp_recv",
                "unreachs",
                "ping",
                "L3D/L4D/L4S",
                "sizes",
                "sa/ra",
                "appflags",
                "port_count",
                "port_sig[port, count]");
        print "$sbuf";
}




$curip = 0;

sub sumportsby_value
{
        if ($sumPS{$curip}->{$a} < $sumPS{$curip}->{$b}) {
                1;
        }
        elsif ($sumPS{$curip}->{$a} == $sumPS{$curip}->{$b}) {
                0;
        }
        elsif ($sumPS{$curip}->{$a} > $sumPS{$curip}->{$b}) {
                -1;
        }
}



sub sumports
{
        my $ipsrc = $_[0];
        my $portkey;
        my $nps="";

        $curip = $ipsrc;

        my @sp  = sort sumportsby_value keys %{$sumPS{$ipsrc}};
        my $ports = scalar @sp;



        # top 10 ports, no more
        if ($ports > 10) {
                $ports = 10;
        }
	
        for (my $i = 0; $i < $ports; $i++) {
                $nps = $nps . "[$sp[$i], $sumPS{$ipsrc}->{$sp[$i]}]";
        }
        print "\tPortuples[$ports]: $nps\n";

}


sub
storeEvent
{
       
        if ($eventflag == 0) {
                return;
        }
        my $eventfile = $_[0];
        my $prependString = $_[1];
        my $eventString = $_[2];
        my $s;

        $eventString=~ s/\n//g;
        $s = "$start_time: " . $prependString . $eventString;

        open(FILE, ">>$eventfile") || die "can't open $eventfile: $!\n";
        print FILE "$s\n";
        close(FILE);
}


sub
udpReport
{
        my $ip = $_[0];
        my $ipdst = $_[1];
        my $weight = $_[2];
        my $udpsent = $_[3];
        my $udprecv = $_[4];
        my $unreach = $_[5];
        my $icmpcount = $_[6];
        my $l3count = $_[7];
        my $l4count = $_[8];
        my $udpflags = $_[9];
        my $ports = $_[10];
        my $size0 = $_[11];
        my $size1 = $_[12];
        my $size2 = $_[13];
        my $size3 = $_[14];
        my $size4 = $_[15];
        my $size5 = $_[16];
        my $scount = $_[17];
        my $portstring = $_[18];
        my $savg = $_[19];
        my $ravg = $_[20];
        my @pset;
        my $pindex;
        my $pcount = 0;
        my @sp;
        my $nps="";
        my $sawpossible_dns = 0;
        my $sawpossible_skype = 0;
        my $guessp2p = 0;
        my $guessstorm = 0;

        my $curport;
        my $curportcount;

      
	#following code is the classifysize routine copied here.
 
       	my $string="";
       	my $s0, $s1, $s2, $s3, $s4, $s5;

       	# convert to percent
       	$s0 = $s1 = $s2 = $s3 = $s4 = $s5 = 0;
       	if ($udpsent) {
               	$s0 = int(($size0*100)/$udpsent);
               	$s1 = int(($size1*100)/$udpsent);
              	$s2 = int(($size2*100)/$udpsent);
               	$s3 = int(($size3*100)/$udpsent);
               	$s4 = int(($size4*100)/$udpsent);
              	$s5 = int(($size5*100)/$udpsent);
       	}
       	if ($s0 > 100) { $s0 = 100;}
       	if ($s1 > 100) { $s1 = 100;}
       	if ($s2 > 100) { $s2 = 100;}
       	if ($s3 > 100) { $s3 = 100;}
       	if ($s4 > 100) { $s4 = 100;}
       	if ($s5 > 100) { $s5 = 100;}
	
       	$sizeattr1 = "$s0/$s1/$s2/$s3/$s4/$s5";
#--------------------------------------------------------
        # format is port,count etc.
        my @ps = split(/,/,$portstring);

        # first compute the total port count
        # meaning: the total number of dst sampled packets.
        # which is NOT necessarily the total number of packets sent.
        for (my $i = 0; $i < $ports; $i++) {
                $pindex = $i * 2;
                $pcount += $ps[$pindex+1];
        }
        # now go thru and compute a percent for the
        # destination port count (hits/total) and store it in an hash
        # array
        for (my $i = 0; $i < $ports; $i++) {
                $pindex = $i * 2;
                if ($pcount) {
                        $pset{$ps[$pindex]} = int(($ps[$pindex+1] * 100.0)/ $pcount);
			#print "$pset{$ps[$pindex]}, $ps[$pindex], $ps[$pindex+1]\n";
                }
        }

       # now sort the array based on the port number itself.
        # sortedPorts is a list of the ports themselves
        @sp  = sort p_by_key keys(%pset);

        # now loop thru the ports
        # putting the ports into one string that is a signature tuple
        for ($i = 0; $i < $ports; $i++) {
                if ($sp[$i] == 53)  {
                        if ($pset{$sp[$i]} > 90) {
                                $sawpossible_dns = 1;
                        }
                }
		if ($pset{$sp[$i]} >= 91) {
			$sawpossible_skype = 1;
		}
                $nps = $nps . "[$sp[$i],$pset{$sp[$i]}]";
                #$ups = $ups . "[$sp[$i]]";                     # ports only
        }

        foreach $key (keys(%pset)){
                delete $pset{$key};
        }
        #$portSignature{$ipsrc} = $nps;         # key is ip, value is ports

        # udp guesstimator here
#############################################################
        # can use sawpossible_dns from above
        # don't know as of yet
        my $guess = "?";

        # guess oneway unless overridden by better info
        if ($udprecv == 0 && $udpsent > 0) {
                $guess = "oneway";
        }
        # scanner defined by maxudpweight
        if ($weight >= $maxudpweight) {
                $guess = "scan";
                if ($l3count >= 10 && $l4count >= 10) {
                        ;
                }
                elsif ($l3count > 10) {
                        $guess = "ipscan";
                }
                elsif ($l4count > 10) {
                        $guess = "portscan";
                }
		# on the other hand 
		if ($l3count >= 10 && $l4count >= 10) {
			# problem - sometimes p2p looks like scanning					# due to a high work weight
			# it is dicey to use a count here for recv but
			# the point is that a lot of recv packets means
			# one is not a scanner so much
			if ($udprecv > 500) {
                        	$guess = "scan/p2p?";
			}
		}
        }
        else {
		# if p2p
		if ($l3count >= 10 && $l4count >= 10) {
			# and not dns
                        if ($sawpossible_dns == 0) {
                               $guess = "p2p";
                                # Added by Divya Parekh on March 9th 2008 for detecting different types of p2p apps
                                $guess .= &p2p_detect($s0, $s1, $s2, $s3, $savg, $ravg, $l3count, $l4count);

                                # if p2p and possible stormworm put out advisory event
                                if ($guess =~ m/St/) {  #if $guess contains St, its a storm-worm!!
                                        $guessstorm = 1;
                                }
                                else {
                                    $guessp2p = 1;
                                }
                        }
                        # maybe DNS but maybe not - we really need L7 info
                        else {
                                if ($savg < $ravg) {
                                        $guess = "DNS?";
                                }
                        }
                }
        }

	# Added by Divya Parekh on May 4th 2008 for detecting skype
	if ($sawpossible_skype == 1) {
		$guess .= &skype_detect($s0, $s1, $s2, $s3, $savg, $ravg);
	}

# end udp guesstimator
#############################################################

        # print it out
        my $dstring = "$l3count" . "\/" . "$l4count" . "\/" . "$scount";
        my $sizestring2 = "$savg" . "\/" . "$ravg";

       	my $sbuf = sprintf(
                	"%-16.16s %-16.16s %-12.12s %-16.16s %9d %9d %9d %9d %14s %-16.16s %-7.7s   %-12.12s      %3d: %s",
                	$ip,  $ipdst,
                	$weight, $guess, $udpsent, $udprecv, $unreach, $icmpcount,
                        $dstring, $sizeattr1, $sizestring2, $udpflags, $ports, $nps);

        if ($mode_30sec == 1){ 
		if ($guessstorm) {
                	storeEvent($eventfile,"possible stormworm:\n", $sbuf);
		}
        	print "$sbuf\n";
		return($sbuf);
	}
	else {  

#Guesstimator code used 
################################################################
		# summarisation mode 
                if($p2p_only == 1 && $guessp2p == 0 && $guessstorm == 0) {
		          return "";
		          # since it is filter p2p only 
                }
###############################################################

                my $pindex = 0;
		#print "PORTS number are $ports\n"; 
                for ($i = 0; $i < $ports; $i++) {
			
                        $pindex = $i * 2;
                        $curport = $ps[$pindex];        # the port
                        $curportcount = $ps[$pindex+1]; # the pkt count
			#print"curport is $curport and value is $curportcount\n";
                        if ($sumPS{$ip}->{$curport}) {
                                $sumPS{$ip}->{$curport} += $curportcount;
                        }
                        else {
                                $sumPS{$ip}->{$curport} = $curportcount;
                        }
                }

		if ($sumInstances{$ip}) {
			$sumInstances{$ip}++ ;

                	#for summing the dst ips    $sumIpdst{$ip}
			my $val=0;
			foreach $dstkey (keys %{$sumIpdst{$ip}}) {
				 if ($dstkey eq $ipdst) {
					$sumIpdst{$ip}->{$dstkey} += 1;
					$val=1;
				 }
			}
			if ($val==0) {
				$sumIpdst{$ip}->{$ipdst}=1;
			}
		
			my $uapplh = hex($uapplc);	
			#for b
			if ($uapplh & 0x2 ){
				$f="b";
				if($sumFlag{$ip}->{$f}) {
					$sumFlag{$ip}->{$f} +=1;
				}
				else {
					$sumFlag{$ip}->{$f} = 1;
				}
			}
		        #for s flag	
			if ($uapplh & 0x40000){
                                $f="s";
                                if($sumFlag{$ip}->{$f}) {
                                        $sumFlag{$ip}->{$f} +=1;
                                }
                                else {
                                        $sumFlag{$ip}->{$f} = 1;
                                }
                        }
	
			# for d		
			if ($uapplh & 0x8 ){
				$f="d";

				if($sumFlag{$ip}->{$f}) {
					$sumFlag{$ip}->{$f} +=1;
				}
				else {
					$sumFlag{$ip}->{$f} = 1;
				}
			}

			# for g		
			if ($uapplh & 0x40){
				$f="g";
				if($sumFlag{$ip}->{$f}) {
					$sumFlag{$ip}->{$f} +=1;
				}
				else {
					$sumFlag{$ip}->{$f} = 1;
				}
			}
				 
			$sumUdpsent{$ip} += $udpsent;
			$sumUdprecv{$ip} += $udprecv;
                	$sumIcmpcount{$ip} += $unreach;
                	$sumPorts{$ip} += $ports;
			$sumL3count{$ip} += $l3count;
			$sumL4count{$ip} += $l4count;
			$sumScount{$ip} += $scount;
                	$sumSavg{$ip} += $savg;
                	$sumRavg{$ip} += $ravg;
                	$sumSize0{$ip} += $s0;
                	$sumSize1{$ip} += $s1;
                	$sumSize2{$ip} += $s2;
                	$sumSize3{$ip} += $s3;
                	$sumSize4{$ip} += $s4;
                	$sumSize5{$ip} += $s5;
                	$sumLastTS{$ip} = $cur_ts;

#Guesstimator code used 
################################################################
            		if ($guessp2p == 1){
				if ($sumGuessp2p{$ip}) {
                        		$sumGuessp2p{$ip}++;
                        	}
                        	else {
                                	$sumGuessp2p{$ip}=1;
                        	}
            		}
            		elsif ($guessstorm == 1) {
				if ($sumGuessp2p{$ip}) {
					$sumGuessp2p{$ip}++;
				}
				else {
					$sumGuessp2p{$ip}=1;
				}
##Preparing the security list for storm worm
				$sumStrmv[$c2] = $sbuf;
				$c2 += 1;
                 	}
	     		elsif ($guess eq "oneway"){
				if ($sumGuessoneway{$ip}) {
					$sumGuessoneway{$ip}++;
				}
				else {
					$sumGuessoneway{$ip}=1;
				}
			}
			elsif (($guess eq "ipscan")||($guess eq "portscan")||($guess eq "scan")) {
				#print "$ip has scan\n";
				if ($sumGuessscan{$ip}){
					 $sumGuessscan{$ip}++;
				}
				else {
					$sumGuessscan{$ip}=1;
				}
			} 
#################################################################################
	                $sumWorkweight{$ip} += $weight;   # for computing average

 			 # keep max and min workweights seen
                	if ($weight > $sumWorkweightMax{$ip}) {
                        	$sumWorkweightMax{$ip} = $weight;
                	}
                	if ($weight < $sumWorkweightMin{$ip}) {
                        	$sumWorkweightMin{$ip} = $weight;
                	}
		}
                else {
			####### for first time only
			$sumGuessp2p{$ip}=0;
			$sumGuessoneway{$ip}=0;
			$sumGuessscan{$ip}=0;
 	               	$sumInstances{$ip} = 1;
			$sumIpdst{$ip}->{$ipdst}=1;
		
			$sumFlag{$ip}->{"b"}=0;
			$sumFlag{$ip}->{"d"}=0;
			$sumFlag{$ip}->{"g"}=0;
			$sumFlag{$ip}->{"s"}=0;
			my $uapplh = hex($uapplc);
			if($uapplh & 0x2) {
				$sumFlag{$ip}->{"b"}=1;
			}
			if($uapplh & 0x40000) {
				$sumFlag{$ip}->{"s"}=1;
			}
			if($uapplh & 0x8) {
				
				$sumFlag{$ip}->{"d"}=1;
			}
			if($uapplh & 0x40) {
				
				$sumFlag{$ip}->{"g"}=1;
			}

                	$sumUdpsent{$ip} = $udpsent;
                	$sumUdprecv{$ip} = $udprecv;
                	$sumIcmpcount{$ip} = $unreach;
                	$sumPorts{$ip} = $ports;
                	$sumL3count{$ip}=$l3count;
                	$sumL4count{$ip}=$l4count;
                	$sumScount{$ip}=$scount;
                	$sumSavg{$ip}=$savg;
                	$sumRavg{$ip}=$ravg;
               	        $sumSize0{$ip}=$s0;
                	$sumSize1{$ip}=$s1;
                	$sumSize2{$ip}=$s2;
                	$sumSize3{$ip}=$s3;
                	$sumSize4{$ip}=$s4;
                	$sumSize5{$ip}=$s5;

#Guesstimator code used 
################################################################
			if ($guessp2p == 1){
				$sumGuessp2p{$ip} = 1;
			}
		##Preparing the security list for storm worm
			elsif ($guessstorm == 1) {
				$sumGuessp2p{$ip} = 1;
				$sumStrmv[$c2] = $sbuf;
				$c2 += 1;
			}              
			elsif ($guess eq "oneway") {
				$sumGuessoneway{$ip} = 1 ;
			}
			elsif (($guess eq "ipscan")||($guess eq "portscan")||($guess eq "scan")) {
			       $sumGuessscan{$ip} = 1;
			}
######################################################################### 
			$sumFirstTS{$ip} = $cur_ts;
			$sumudpflags{$ip} =$udpflags;           

			$sumWorkweight{$ip} = $weight;   # for computing average
			$sumWorkweightMax{$ip} = $weight;
			$sumWorkweightMin{$ip} = $weight;
		}
		$sumAvgww{$ip}= $sumWorkweight{$ip}/$sumInstances{$ip};

		return "";
	}
}

# insubnet
#
# note: input param netmask may have these forms
#       1.0.0.0/8
#       1.0.0.0/8,2.0.0.0/8      
#       1/8
#       10.10/16
#       1/8,2/8,200.1.2.0/24
#
# a comma-separated list is possible.
# so called netmask variable below is really net/mask.
#
# compare a subnet/mask to an ip
# return:
#       1 if ip within subnet/s
#       0 otherwise
                                
sub insubnet
{
        my $netmask = $_[0];     
        my $ip = $_[1];
        my $hexmask;
        my $rnet1;
        my $rnet2;
        my $net1;
        my $net2;
        my $index=0;
       
        my @nets = split(",", $netmask);

        while ($nets[$index]) {

                my @words = split("\/", $nets[$index]);
                my $subnet = $words[0];
                my $mask = $words[1];

                # fix subnet if too short - we need full dot quad notation
                my @quads=split('\.',$subnet);
                my $suffixquads = 3 - $#quads;  # number of .zero quads needed at end
                # append enough dot quads if needed
                if ($suffixquads == 3) {
                        $subnet = $subnet . ".0.0.0";
                }
                elsif ($suffixquads == 2) {
                        $subnet = $subnet . ".0.0";
                }
                elsif ($suffixquads == 1) {
                        $subnet = $subnet . ".0";
                }

                # if slash notation
                # 1. convert the specified mask
                # to a hex value (a real mask)
                $hexmask = 0xffffffff << (32 - $mask);
                # limit to 32 bits please
                $hexmask = $hexmask & 0xffffffff;

                # ip address based netmask as opposed to slash
                #$hexmask = unpack('N', inet_aton($mask));

                # 2. convert the ipaddr to a host int
                $net1 = unpack('N', inet_aton($ip));
                $net2 = unpack('N', inet_aton($subnet));
                # printf ("inputip: %x inputnet %x mask %x\n", $net1, $net2, $hexmask);

                # 3. compare the results
                $rnet1 = $net1 & $hexmask;
                $rnet2 = $net2 & $hexmask;

                # print "$rnet1, $rnet2:", "$rnet1 == $rnet2\n";
                if ($rnet1 == $rnet2) {
                        return (1);
                }
                $index++;
        }
        return(0);
}





# weak check to see if timestamp is in name
sub check_ts
{
        my @DAYS=
        my $candidate_ts = $_[0];
        my @TSPARTS = split("_", $candidate_ts);
        if ($TSPARTS[0] eq "Sun") {
                return(1);
        }
        if ($TSPARTS[0] eq "Mon") {
                return(1);
        }
        if ($TSPARTS[0] eq "Tue") {
                return(1);
        }
        if ($TSPARTS[0] eq "Wed") {
                return(1);    
        }
        if ($TSPARTS[0] eq "Thu") {
                return(1);
        }
        if ($TSPARTS[0] eq "Fri") {
                return(1);
        }
        if ($TSPARTS[0] eq "Sat") {
                return(1);
        }
        return(0);
}



#internal dns cache if reverse dns in use.
#
# return dns from ip within timeout period of 5 seconds
# dnscache{} is associative array so that we can
#       cache our own dns lookups
sub gethostip {
        my $ipaddr = $_[0];
        my $dnsname;

        if ($dnscache{$ipaddr}) {
                if ($debug) {
                        print "cached $ipaddr: $dnscache{$ipaddr}\n";
                }
                return ($dnscache{$ipaddr});
        }


        if ($debug) {
                print "gethostip: lookup of $ipaddr\n";
        }
        eval {
                $SIG{ALRM} = \&sigalrm;
                alarm(5);
                eval {
                        $dnsname = gethostbyaddr(inet_aton($ipaddr), 2);
                };
                alarm(0);
        };
        alarm(0);
#print "ip: $ipaddr dns: $dnsname\n";
        if ($dnsname) {
                $dnscache{$ipaddr} = $dnsname;
                if ($debug) {
                        print "gethostip: found $dnsname\n";
                }
        }
        else {
                $dnscache{$ipaddr} = $ipaddr;
                if ($debug) {
                        print "gethostip: did not find $ipaddr\n";
                }
        }
        return($dnsname);
}


# sort by the key for the ports array
# above
sub p_by_key
{
        if ($a < $b) {
                -1;
        }
        elsif ($a == $b) {
                0;
        }
        elsif ($a > $b) {
                1;
        }
}




# parse syn_list app flags
sub get_appflag
{
        my $appflags = $_[0];
        my $apptagsLower = $_[1];
        my $apptagsUpper = $_[2];
        my $emulehits = 1;

        my $af = parseAppFlags(hex($appflags), $emulehits);
        $af = parseLowerAppTags(hex($apptagsLower),$af);
        $af = parseUpperAppTags(hex($apptagsUpper),$af);
        return($af);
}

# ignoring emulehit count for now
sub parseAppFlags
{
        $appflags = $_[0];
        $emulehits = $_[1];
        #$appflags = hex($_[0]);
        my $string="";

        if ($appflags & 0x0f) {
                $string = "I";          #/* irc of some sort */
        }
        if ($appflags & 0x10) {
                $string .= "B";         #/* bittorrent */
        }
        if ($appflags & 0x20) {
                $string .= "G";         #/* GNUTELLA */
        }
        if ($appflags & 0x40) {
                $string .= "K";         #/* KAZAA */
        }
        if ($appflags & 0x80 && ($emulehits >= $emulecount)) {
                $string .= "e";         #/* edonkey */
        }
        if ($appflags & 0x100) {
                $string .= "P";         #/* honeypot dst */
        }
        if ($appflags & 0x200) {
                $string .= "M";         #/* Morpheus */
        }
        if ($appflags & 0x400) {
                $string .= "H";         #/* web server based on tcp src port */
        }
        if ($appflags & 0x800) {
                $string .= "E";         # email based on tcp dst port
        }
        if ($appflags & 0x1000) {
                $string .= "D";         # darknet dst
        }
        if ($appflags & 0x2000) {
                $string .= "X";         # evil flag
                $evilflag = 1;
        }
        return($string);
}


sub parseLowerAppTags
{
        $apptag = $_[0];
        $string = $_[1];

        if ($apptag & 0x1 ) { $string .= "a"; }
        if ($apptag & 0x2 ) { $string .= "b"; }
        if ($apptag & 0x4 ) { $string .= "c"; }
        if ($apptag & 0x8 ) { $string .= "d"; }
        if ($apptag & 0x10 ) { $string .= "e"; }
        if ($apptag & 0x20 ) { $string .= "f"; }
        if ($apptag & 0x40 ) { $string .= "g"; }
        if ($apptag & 0x80 ) { $string .= "h"; }
        if ($apptag & 0x100 ) { $string .= "i"; }
        if ($apptag & 0x200 ) { $string .= "j"; }
        if ($apptag & 0x400 ) { $string .= "k"; }
        if ($apptag & 0x800 ) { $string .= "l"; }
        if ($apptag & 0x1000 ) { $string .= "m"; }
        if ($apptag & 0x2000 ) { $string .= "n"; }
        if ($apptag & 0x4000 ) { $string .= "o"; }
        if ($apptag & 0x8000 ) { $string .= "p"; }
        if ($apptag & 0x10000 ) { $string .= "q"; }
        if ($apptag & 0x20000 ) { $string .= "r"; }
        if ($apptag & 0x40000 ) { $string .= "s"; }
        if ($apptag & 0x80000 ) { $string .= "t"; }
        if ($apptag & 0x100000 ) { $string .= "u"; }
        if ($apptag & 0x200000 ) { $string .= "v"; }
        if ($apptag & 0x400000 ) { $string .= "w"; }
        if ($apptag & 0x800000 ) { $string .= "x"; }
        if ($apptag & 0x1000000 ) { $string .= "y"; }
        if ($apptag & 0x2000000 ) { $string .= "z"; }

        return($string);
}




sub parseUpperAppTags
{
        $apptag = $_[0];
        $string=  $_[1];

        if ($apptag & 0x1 ) { $string .= "A"; }
        if ($apptag & 0x2 ) { $string .= "B"; }
        if ($apptag & 0x4 ) { $string .= "C"; }
        if ($apptag & 0x8 ) { $string .= "D"; }
        if ($apptag & 0x10 ) { $string .= "E"; }
        if ($apptag & 0x20 ) { $string .= "F"; }
        if ($apptag & 0x40 ) { $string .= "G"; }
        if ($apptag & 0x80 ) { $string .= "H"; }
        if ($apptag & 0x100 ) { $string .= "I"; }
        if ($apptag & 0x200 ) { $string .= "J"; }
        if ($apptag & 0x400 ) { $string .= "K"; }
        if ($apptag & 0x800 ) { $string .= "L"; }
        if ($apptag & 0x1000 ) { $string .= "M"; }
        if ($apptag & 0x2000 ) { $string .= "N"; }
        if ($apptag & 0x4000 ) { $string .= "O"; }
        if ($apptag & 0x8000 ) { $string .= "P"; }
        if ($apptag & 0x10000 ) { $string .= "Q"; }
        if ($apptag & 0x20000 ) { $string .= "R"; }
        if ($apptag & 0x40000 ) { $string .= "S"; }
        if ($apptag & 0x80000 ) { $string .= "T"; }
        if ($apptag & 0x100000 ) { $string .= "U"; }
        if ($apptag & 0x200000 ) { $string .= "V"; }
        if ($apptag & 0x400000 ) { $string .= "W"; }
        if ($apptag & 0x800000 ) { $string .= "X"; }
        if ($apptag & 0x1000000 ) { $string .= "Y"; }
        if ($apptag & 0x2000000 ) { $string .= "Z"; }

        return($string);
}



# returns one string with all sizes
sub classifySize
{
        $size0=$_[0];
        $size1=$_[1];
        $size2=$_[2];
        $size3=$_[3];
        $size4=$_[4];
        $size5=$_[5];
        $udpsent=$_[6];
        my $string="";
        my $s0, $s1, $s2, $s3, $s4, $s5;

        #return("$size0/$size1/$size2/$size3");
        # convert to percent
        $s0 = $s1 = $s2 = $s3 = $s4 = $s5 = 0;
        if ($udpsent) {
                $s0 = int(($size0*100)/$udpsent);
                $s1 = int(($size1*100)/$udpsent);
                $s2 = int(($size2*100)/$udpsent);
                $s3 = int(($size3*100)/$udpsent);
                $s4 = int(($size4*100)/$udpsent);
                $s5 = int(($size5*100)/$udpsent);
        }
        if ($s0 > 100) { $s0 = 100;}
        if ($s1 > 100) { $s1 = 100;}
        if ($s2 > 100) { $s2 = 100;}
        if ($s3 > 100) { $s3 = 100;}
        if ($s4 > 100) { $s4 = 100;}
        if ($s5 > 100) { $s5 = 100;}
        # returns one string
        return("$s0/$s1/$s2/$s3/$s4/$s5");
}


# p2p application detector here
#############################################################
sub p2p_detect
{
  $s0 = $_[0];
  $s1 = $_[1];
  $s2 = $_[2];
  $s3 = $_[3];
  $savg = $_[4];
  $ravg = $_[5];	
  $l3count = $_[6];	
  $l4count = $_[7];	

  my $myguess = "";

# Emule KAD & servers
	if (($s0 >= 30 && $s0 <= 100)		#14
	 && ($s1 >= 0  && $s1 <= 15)		#20 16 73
	 && ($s2 >= 1  && $s2 <= 40)		#0 0 0 11
	 && ($s3 >= 0  && $s3 <= 10)
	 && ($savg >= 25 && $savg <= 70)	#39 26 29 29
	 && ($ravg >= 30 && $ravg <= 85))	#92 93 23 21 25
	{
		$myguess .= ":eKS";
	} 

# Emule KAD only
	if (($s0 >= 30 && $s0 <= 90)		#100
	 && ($s1 >= 0  && $s1 <= 55)		#53
	 && ($s2 >= 0  && $s2 <= 30)
	 && ($s3 >= 0  && $s3 <= 5)
	 && ($savg >= 40 && $savg <= 70)	#30 39 39
	 && ($ravg >= 0 && $ravg <= 85))	#87 91
	{
		$myguess .= ":eK";
	} 

# Emule Servers only
	if (($s0 >= 0  && $s0 <= 20) 
	 && (($s1 >= 10 && $s1 <= 30) || ($s1 >= 80 && $s1 <= 100))
	 && ($s2 >= 0  && $s2 <= 10)
	 && ($s3 == 0  || ($s3 >= 70 && $s3 <= 90))
	 && (($savg >= 10 && $savg <= 30) || ($savg >= 80 && $savg <= 100))
	 && (($ravg >= 10 && $ravg <= 30) || ($ravg >= 80 && $ravg <= 100)))
	{
		$myguess .= ":eS";
	} 

# Bittorent Azureus
	if (($s0 >= 0 && $s0 <= 60) 
	 && ($s1 >= 20 && $s1 <= 97)		#
	 && ($s2 >= 0  && $s2 <= 18)		#20
	 && ($s3 >= 1  && $s3 <= 20)		#
	 && ($savg >= 50 && $savg <= 90)	#45
	 && ($ravg >= 60 && $ravg <= 136))	#151
	{
		$myguess .= ":bA";
	} 

# Bittorent utorrent
	if (($s0 >= 0 && $s0 <= 3)
	 && ($s1 >= 10 && $s1 <= 90)		#5 1
	 && ($s2 >= 0  && $s2 <= 40)		#64 82 91 41 42
	 && ($s3 >= 10 && $s3 <= 100)		#0 2 3 6 5 4 8
	 && ($savg >= 80 && $savg <= 200)	#72 78
	 && ($ravg >= 41 && $ravg <= 200))	#3 30 34
	{
		$myguess .= ":bT";
	} 

# Bittorent Bittorent
	if (($s0 == 0)
	 && ($s1 >= 1  && $s1 <= 94)		#
	 && ($s2 >= 1  && $s2 <= 30)		#39 83 0 93 48 55 65 80 62
	 && ($s3 >= 10  && $s3 <= 80)		#1 5 6 7 9 1 1 6 8
	 && ($savg >= 94 && $savg <= 190)	#80 70 69 74 80 76 75 81 86
	 && ($ravg >= 60 && $ravg <= 160))	#54 179 51 54 51 53 40 50
	{
		$myguess .= ":bB";
	} 

# Gnutella Limewire
	if (($s0 >= 0 && $s0 <= 30)
	 && ($s1 >= 20 && $s1 <= 80) 
	 && ($s2 >= 0 && $s2 <= 50)
	 && ($s3 >= 0 && $s3 <= 50)
	 && (($savg >= 41 && $savg <= 100) || ($savg >= 130 && $savg <= 150))
	 && ($ravg >= 70 && $ravg <= 190))
	{
		$myguess .= ":gL";
	} 

# Joost
	if (($s0 >= 30 && $s0 <= 80) 
	 && ($s1 >= 1  && $s1 <= 40) 
	 && ($s2 >= 0  && $s2 <= 35)
	 && ($s3 >= 0  && $s3 <= 50)		#61 59 51 93
	 && ($savg >= 20 && $savg <= 135)	#162 162 141
	 && ($ravg >= 180 && $ravg <= 210))	#106 #19
	{
		$myguess .= ":Jo";
	} 

# Stormworm
	if (($s0 >=98)
	 && ($s1 == 0) 
	 && ($s2 == 0)
	 && ($s3 == 0)
	 && ($savg >= 20 && $savg <= 27)
	 && ($ravg >= 140 && $ravg <= 240))
	{
		# make sure it really is somewhat p2p
		if ($l3count > 40 && $l4count > 40) {
			$myguess .= ":St";
		}
	} 

	return ($myguess);
};
# end p2p application detector
#############################################################

# skype detector here
#############################################################
sub skype_detect
{
	$s0 = $_[0];
	$s1 = $_[1];
	$s2 = $_[2];
	$s3 = $_[3];
	$savg = $_[4];
	$ravg = $_[5];	
	my $skyguess = "";

 	if (($s2 > $s0 && $s2 > $s1 && $s2 > $s3)
# was 1, now 0
     		&& ($s0 >= 0 && $s0 <= 61)
 		    && ($s1 >= 0 && $s1 <= 13)
 		    && ($s2 >= 50 && $s2 <= 100)
 		    && ($s3 >= 0 && $s3 <= 10)
# was 110, now 130
 		    && ($savg >= 70 && $savg <= 130)	
# was 0100..325
 		    && ($ravg >= 95 && $ravg <= 325)) {
 		$skyguess = ":sky";
 	} 
	return ($skyguess);
} 
# end skype detector
#############################################################
